/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2004 by Ralf Baechle
 */
#ifndef __ASM_PPC_MARVELL_H
#define __ASM_PPC_MARVELL_H

#include <asm/byteorder.h>
#include <asm/system.h>

extern unsigned long marvell_base;

/*
 * Because of an error/peculiarity in the Galileo chip, we need to swap the
 * bytes when running bigendian.
 */
/*
 * Nicolas DET:
 * On PowerPC (especially for G4) we have to be careful about the re-ordering
 * as aresult I add some mb() in the MV_xxxx macros
*/
#define __MV_READ(ofs)								\
	(*(volatile u32 *)(marvell_base+(ofs)))					

#define __MV_WRITE(ofs, data)							\
	do { *(volatile u32 *)(marvell_base+(ofs)) = (data); } while (0)	

#if 1
extern inline u32 mv_read(u32 offset)
{
	u32 RegVal = le32_to_cpu(__MV_READ(offset));
	rmb();
	return RegVal;
}

extern inline void mv_write(u32 offset, u32 RegVal)
{
	__MV_WRITE(offset, cpu_to_le32(RegVal));
	wmb();
}

#define MV_READ(ofs) 		mv_read( (u32) ofs)
#define MV_WRITE(ofs, data) 	mv_write( (u32) ofs, (u32) data)

#else
#define MV_READ(ofs)		le32_to_cpu(__MV_READ(ofs))
#define MV_WRITE(ofs, data)	__MV_WRITE(ofs, cpu_to_le32(data))
#endif

#define MV_SET_REG_BITS(ofs, bits)					\
	(*((volatile u32 *)(marvell_base + (ofs)))) |= ((u32)cpu_to_le32(bits))
#define MV_RESET_REG_BITS(ofs, bits)					\
	(*((volatile u32 *)(marvell_base + (ofs)))) &= ~((u32)cpu_to_le32(bits))

#endif	/* __ASM_PPC_MARVELL_H */
